@@ -8,8 +8,8 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -1215,6 +1215,9 @@ void copycat()
       (*newobj)->viewscale = oldobj->viewscale;
       /* don't attach the same schematic. . . */
       (*newobj)->symschem = NULL;
+      /* don't copy highlights */
+      (*newobj)->highlight.netlist = NULL;
+      (*newobj)->highlight.thisinst = NULL;
 
       /* Copy the parameter structure */
       (*newobj)->params = NULL;
@@ -1583,6 +1586,7 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
 
    areastruct.topinstance = xobjs.libtop[libmod];
    setpage(TRUE);
+   invalidate_graphics(topobject);
 
    /* draw the new screen */
 
