@@ -15,6 +15,7 @@ extern void truncate_undo_stack();
 extern void free_undo_record(Undoptr);
 extern void free_redo_record(Undoptr);
 extern stringpart *get_original_string(labelptr);
+extern short *recover_selectlist(Undoptr);
 #ifndef TCL_WRAPPER
 extern void undo_call(xcWidget, caddr_t, caddr_t);
 extern void redo_call(xcWidget, caddr_t, caddr_t);
@@ -34,6 +35,22 @@ extern char *TCL_to_PS(char *);
 
 extern char *evaluate_expr(oparamptr, objinstptr);
 
+/* from opengl.c:  compatibility functions for OpenGL/X11 */
+
+#ifdef OPENGL
+extern void SetForeground(Display *, GC, int);
+extern void SetBackground(Display *, GC, int);
+extern void SetFunction(Display *, GC, int);
+extern void SetLineAttributes(Display *, GC, float, int, int, int);
+extern void SetDashes(Display *, GC, int, char dashlist[], int);
+extern void SetStipple(Display *, GC, int);
+extern void SetFillStyle(Display *, GC, int);
+extern void DrawLine(Display *, Window, GC, int, int, int, int);
+extern void DrawLines(Display *, Window, GC, XPoint *, int, int);
+extern void DrawPoint(Display *, Window, GC, int, int);
+extern void FillPolygon(Display *, Window, GC, XPoint *, int, int, int);
+#endif
+
 /* from elements.c: */
 
 /* element constructor functions */
@@ -165,7 +182,7 @@ extern void startedit(xcWidget, caddr_t, caddr_t);
 extern void startdelete(xcWidget, caddr_t, caddr_t);
 extern void reviseselect(short *, int, short *);
 extern void deletebutton(XButtonEvent *);
-extern short *xc_undelete(objinstptr, objectptr, short);
+extern short *xc_undelete(objinstptr, objectptr, short, short *);
 extern objectptr delete_element(objinstptr, short *, int, short);
 extern void printname(objectptr);
 extern Boolean checkname(objectptr);
@@ -313,7 +330,7 @@ extern short checkbounds();
 extern void window_to_user(short, short, XPoint *);
 extern void user_to_window(XPoint, XPoint *);
 extern float UTopScale();
-extern short UTopTransScale(float);
+extern float UTopTransScale(float);
 extern XPoint UGetCursor();
 extern XPoint UGetCursorPos();
 extern void u2u_snap(XPoint *);
@@ -665,6 +682,13 @@ extern void loadrcfile();
 extern short readcommand(short, FILE *);
 #endif
 
+/* from graphic.c */
+
+extern void UDrawGraphic(graphicptr);
+extern graphicptr new_graphic(objinstptr, char *, int, int);
+extern void invalidate_graphics(objectptr);
+extern void freegraphic(graphicptr);
+
 /* from render.c: */
 
 extern void ghostinit();
@@ -717,7 +741,7 @@ extern Boolean schemassoc(objectptr, objectptr);
 
 extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
-extern void geneasydraw(short, short, objectptr, objinstptr);
+extern void geneasydraw(short, int, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
 extern selection *genselectelement(short, u_char, objectptr, objinstptr);
 extern short *allocselect();
