@@ -101,7 +101,7 @@ void reviseselect(short *slist, int selects, short *removed)
 /* Draw a selected item */
 /*----------------------*/
 
-void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curinst)
+void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 {
    genericptr elementptr = *(curobj->plist + instance);
 
@@ -133,16 +133,16 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
 
 void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
 {
-   XSetFunction(dpy, areastruct.gc, GXcopy);
-   XSetForeground(dpy, areastruct.gc, BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXcopy);
+   SetForeground(dpy, areastruct.gc, BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
    indicateparams(*(curobj->plist + *newselect));
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
    indicateparams(*(curobj->plist + *newselect));
 }
 
@@ -382,13 +382,32 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 
 	 labelbbox(curlab, newboxpts, selinst);
 
-         /* check for point inside bounding box, as for objects */
+	 /* Need to check for zero-size boxes or test_insideness()	*/
+	 /* fails.  Zero-size boxes happen when labels are parameters	*/
+	 /* set to a null string.					*/
 
-	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+	 if ((newboxpts[0].x != newboxpts[1].x) || (newboxpts[0].y !=
+		newboxpts[1].y)) {
+	
+            /* check for point inside bounding box, as for objects */
+
+	    selected = test_insideness(areastruct.save.x, areastruct.save.y,
 		newboxpts);
-	 if (selected) textpos = textend = 0;
+	    if (selected) textpos = textend = 0;
+	 }
       }
 
+      else if ((*curgen)->type == (class & GRAPHIC)) {
+
+         /* Look among the graphic images */
+
+	 graphicptr curg = TOGRAPHIC(curgen);
+	 graphicbbox(curg, newboxpts);
+
+         /* check for point inside bounding box, as for objects */
+	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+		newboxpts);
+      }
 
       else if ((*curgen)->type == (class & PATH)) {
 
@@ -532,13 +551,21 @@ void selectarea()
 
       switch(ELEMENTTYPE(*curgen)) {
 	case(OBJINST):
-           /* check for instance of object inside area box */
+           /* check for instance of object center point inside area box */
            selected = (TOOBJINST(curgen)->position.x < areastruct.save.x) &&
 		(TOOBJINST(curgen)->position.x > areastruct.origin.x) &&
 		(TOOBJINST(curgen)->position.y < areastruct.save.y) &&
 		(TOOBJINST(curgen)->position.y > areastruct.origin.y);
 	   break;
 
+	case(GRAPHIC):
+           /* check for graphic image center point inside area box */
+           selected = (TOGRAPHIC(curgen)->position.x < areastruct.save.x) &&
+		(TOGRAPHIC(curgen)->position.x > areastruct.origin.x) &&
+		(TOGRAPHIC(curgen)->position.y < areastruct.save.y) &&
+		(TOGRAPHIC(curgen)->position.y > areastruct.origin.y);
+	   break;
+
         case(LABEL): {
 	   XPoint bboxpts[4], newboxpts[4], adj;
 	   labelptr slab = TOLABEL(curgen);
@@ -698,7 +725,7 @@ void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 
    /* reset the graphics state */
 
-   XSetFunction(dpy, areastruct.gc, GXcopy);
+   SetFunction(dpy, areastruct.gc, GXcopy);
 
    for (lastselect = areastruct.selectlist; lastselect < areastruct.selectlist 
 	+ areastruct.selects; lastselect++) {
@@ -1047,7 +1074,7 @@ short *recurse_select_element(short class, u_char mode) {
       }
       if (ilast >= 0) {
 	 newselect = rselect->selectlist + ilast;
-         XSetFunction(dpy, areastruct.gc, GXcopy);
+         SetFunction(dpy, areastruct.gc, GXcopy);
          XTopSetForeground(GSELTOCOLOR(topobject, newselect));
          geneasydraw(*newselect, DEFAULTCOLOR, topobject, areastruct.topinstance);
          areastruct.selects--;
