@@ -14,6 +14,35 @@ typedef unsigned int	u_int;
 typedef unsigned long	u_long;
 #endif
 
+/*----------------------------------------------------------------------*/
+/* Graphics functions defined for X11 (i.e., not OpenGL)		*/
+/*----------------------------------------------------------------------*/
+
+#ifndef OPENGL
+
+#define DrawLine	XDrawLine
+#define DrawLines	XDrawLines
+#define DrawPoint	XDrawPoint
+#define FillPolygon	XFillPolygon
+
+#define SetForeground	XSetForeground
+#define SetBackground	XSetBackground
+#define SetFunction	XSetFunction
+#define SetThinLineAttributes	XSetLineAttributes
+#define SetLineAttributes(a, b, c, d, e, f) \
+	 XSetLineAttributes(a, b, ((c) >= 1.55 ? (int)(c + 0.45) : 0), d, e, f)
+#define SetDashes	XSetDashes
+#define SetFillStyle	XSetFillStyle
+#define SetStipple(a,b,c) XSetStipple(a,b,STIPPLE[c])
+#define flusharea()	
+
+#else
+
+#define SetThinLineAttributes	SetLineAttributes
+#define flusharea	glFlush
+
+#endif /* !OPENGL */
+
 /*----------------------------------------------------------------------*/
 /* Redefinition of fprintf() allows redirection of output to a console	*/
 /* in the Tcl interpreter-based version.				*/
@@ -616,8 +645,10 @@ typedef struct {
    XPoint	  position;
    short	  rotation;
    float	  scale;
-   Pixmap	  source;	/* source data */
-   Pixmap	  target;	/* target (scaled) data */
+   XImage	  *source;	/* source data */
+   XImage	  *target;	/* target (scaled) data */
+   Pixmap	  clipmask;	/* clipmask for non-manhattan rotations */
+   Boolean	  valid;	/* does target need to be regenerated? */
 } graphic, *graphicptr;
 
 /*----------------------------------------------------------------------*/
@@ -1033,14 +1064,14 @@ typedef struct {
 /* Macros for GC color and function handling				*/
 /*----------------------------------------------------------------------*/
 
-#define XTopSetForeground(a) if (a == DEFAULTCOLOR) XSetForeground(dpy, \
-	areastruct.gc, FOREGROUND); else XSetForeground(dpy, areastruct.gc, a)
+#define XTopSetForeground(a) if (a == DEFAULTCOLOR) SetForeground(dpy, \
+	areastruct.gc, FOREGROUND); else SetForeground(dpy, areastruct.gc, a)
 
-#define XSetXORFg(a,b) if (a == DEFAULTCOLOR) XSetForeground(dpy, \
-	areastruct.gc, FOREGROUND ^ b); else XSetForeground(dpy, areastruct.gc,\
+#define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
+	areastruct.gc, FOREGROUND ^ b); else SetForeground(dpy, areastruct.gc,\
 	a ^ b)
 
-#define XcSetFunction(z) XSetFunction(dpy, areastruct.gc, z); \
+#define XcSetFunction(z) SetFunction(dpy, areastruct.gc, z); \
 	areastruct.gctype = z
 
 #define XcSetForeground(z) XTopSetForeground(z); areastruct.gccolor = \
@@ -1105,30 +1136,60 @@ typedef struct _keybinding {
 /*----------------------------------------------------------------------*/
 
 enum {
-   XCF_ENDDATA = -2, XCF_SPACER,
-   XCF_Page, XCF_Justify, XCF_Superscript, XCF_Subscript, XCF_Normalscript,
-   XCF_Nextfont, XCF_Boldfont, XCF_Italicfont, XCF_Normalfont, XCF_Underline,
-   XCF_Overline, XCF_ISO_Encoding, XCF_Halfspace, XCF_Quarterspace,
-   XCF_Special, XCF_TabStop, XCF_TabForward, XCF_TabBackward,
-   XCF_Text_Return, XCF_Text_Delete, XCF_Text_Right, XCF_Text_Left,
-   XCF_Text_Up, XCF_Text_Down, XCF_Text_Split,
-   XCF_Text_Home, XCF_Text_End, XCF_Linebreak, XCF_Parameter,
-   XCF_Edit_Param,XCF_Edit_Break, XCF_Edit_Delete, XCF_Edit_Insert,
-   XCF_Edit_Append, XCF_Edit_Next, XCF_Attach,
-   XCF_Next_Library, XCF_Library_Directory,
-   XCF_Library_Move, XCF_Library_Copy, XCF_Library_Edit, XCF_Library_Delete,
-   XCF_Library_Duplicate, XCF_Library_Hide, XCF_Library_Virtual,
-   XCF_Page_Directory, XCF_Library_Pop, XCF_Virtual,
-   XCF_Help, XCF_Redraw, XCF_View, XCF_Zoom_In, XCF_Zoom_Out, XCF_Pan,
-   XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
-   XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
-   XCF_SnapTo, XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc,
-   XCF_Text, XCF_Exchange, XCF_Copy, XCF_Move, XCF_Join, XCF_Unjoin, XCF_Spline,
-   XCF_Edit, XCF_Undo, XCF_Redo, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
-   XCF_Dotted, XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start,
-   XCF_Finish, XCF_Cancel, XCF_Nothing, XCF_Exit,
-   XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Pin_Global, XCF_Info_Label,
-   XCF_Connectivity, XCF_Sim, XCF_SPICE, XCF_PCB, XCF_SPICEflat,
+   XCF_ENDDATA = -2,		XCF_SPACER  /* -1 */,
+   XCF_Page	 /* 0 */,	XCF_Justify /* 1 */,
+   XCF_Superscript /* 2 */,	XCF_Subscript /* 3 */,
+   XCF_Normalscript /* 4 */,	XCF_Nextfont /* 5 */,
+   XCF_Boldfont /* 6 */,	XCF_Italicfont /* 7 */,
+   XCF_Normalfont /* 8 */,	XCF_Underline /* 9 */,
+   XCF_Overline /* 10 */,	XCF_ISO_Encoding /* 11 */,
+   XCF_Halfspace /* 12 */,	XCF_Quarterspace /* 13 */,
+   XCF_Special /* 14 */,	XCF_TabStop /* 15 */,
+   XCF_TabForward /* 16 */,	XCF_TabBackward /* 17 */,
+   XCF_Text_Return /* 18 */,	XCF_Text_Delete /* 19 */,
+   XCF_Text_Right /* 20 */,	XCF_Text_Left /* 21 */,
+   XCF_Text_Up /* 22 */,	XCF_Text_Down /* 23 */,
+   XCF_Text_Split /* 24 */,	XCF_Text_Home /* 25 */,
+   XCF_Text_End /* 26 */,	XCF_Linebreak /* 27 */,
+   XCF_Parameter /* 28 */,	XCF_Edit_Param /* 29 */,
+   XCF_Edit_Break /* 30 */,	XCF_Edit_Delete /* 31 */,
+   XCF_Edit_Insert /* 32 */,	XCF_Edit_Append /* 33 */,
+   XCF_Edit_Next /* 34 */,	XCF_Attach /* 35 */,
+   XCF_Next_Library /* 36 */,	XCF_Library_Directory /* 37 */,
+   XCF_Library_Move /* 38 */,	XCF_Library_Copy /* 39 */,
+   XCF_Library_Edit /* 40 */,	XCF_Library_Delete /* 41 */,
+   XCF_Library_Duplicate /* 42 */, XCF_Library_Hide /* 43 */,
+   XCF_Library_Virtual /* 44 */, XCF_Page_Directory /* 45 */,
+   XCF_Library_Pop /* 46 */,	XCF_Virtual /* 47 */,
+   XCF_Help /* 48 */,		XCF_Redraw /* 49 */,
+   XCF_View /* 50 */,		XCF_Zoom_In /* 51 */,
+   XCF_Zoom_Out /* 52 */,	XCF_Pan /* 53 */,
+   XCF_Double_Snap /* 54 */,	XCF_Halve_Snap /* 55 */,
+   XCF_Pan_Left /* 56 */,	XCF_Pan_Right /* 57 */,
+   XCF_Pan_Up /* 58 */,		XCF_Pan_Down /* 59 */,
+   XCF_Write /* 60 */,		XCF_Rotate /* 61 */,
+   XCF_Flip_X /* 62 */,		XCF_Flip_Y /* 63 */,
+   XCF_Snap /* 64 */,		XCF_SnapTo /* 65 */,
+   XCF_Pop /* 66 */,		XCF_Push /* 67 */,
+   XCF_Delete /* 68 */,		XCF_Select /* 69 */,
+   XCF_Box /* 70 */,		XCF_Arc /* 71 */,
+   XCF_Text /* 72 */,		XCF_Exchange /* 73 */,
+   XCF_Copy /* 74 */,		XCF_Move /* 75 */,
+   XCF_Join /* 76 */,		XCF_Unjoin /* 77 */,
+   XCF_Spline /* 78 */,		XCF_Edit /* 79 */,
+   XCF_Undo /* 80 */,		XCF_Redo /* 81 */,
+   XCF_Select_Save /* 82 */,	XCF_Unselect /* 83 */,
+   XCF_Dashed /* 84 */,		XCF_Dotted /* 85 */,
+   XCF_Solid /* 86 */,		XCF_Prompt /* 87 */,
+   XCF_Dot /* 88 */,		XCF_Wire /* 89 */,
+   XCF_Start /* 90 */,		XCF_Finish /* 91 */,
+   XCF_Cancel /* 92 */,		XCF_Nothing /* 93 */,
+   XCF_Exit /* 94 */,		XCF_Netlist /* 95 */,
+   XCF_Swap /* 96 */,		XCF_Pin_Label /* 97 */,
+   XCF_Pin_Global /* 98 */,	XCF_Info_Label /* 99 */,
+   XCF_Connectivity /* 100 */,	XCF_Sim /* 101 */,
+   XCF_SPICE /* 102 */,		XCF_PCB /* 103 */,
+   XCF_SPICEflat /* 104 */,
    NUM_FUNCTIONS
 };
 
@@ -1153,6 +1214,17 @@ typedef struct {
    XPoint	pagesize;	/* size of page to print on */
 } Pagedata;
 
+/*----------------------------------------------------------------------*/
+/* Structure holding information about graphic images used.  These hold	*/
+/* the original data for the images, and may be shared.			*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   XImage	*image;
+   int		refcount;
+   char		*filename;
+} Imagedata;
+
 /*----------------------------------------------------------------------*/
 /* The main globally-accessible data structure.  This structure holds	*/
 /* all the critical data needed by the drawing window 			*/
@@ -1261,6 +1333,8 @@ typedef struct {
    Library	fontlib;
    Library	*userlibs;
    objinstptr   *libtop;
+   Imagedata	*imagelist;
+   short	images;
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
